<?php

namespace Drupal\content_packager;

use Drupal\Core\Archiver\Zip;
use Drupal\views\Views;


/**
 * Batch operations to apply to Views for content packaging.
 *
 * @package content_packager
 */
class BatchOperations {

  /**
   * Implements callback_batch_operation().
   *
   * @param string $packageUri
   *   The view rows to process.
   * @param array|\ArrayAccess $context
   *   The batch context array.
   */
  public static function prepareDestination($packageUri, &$context) {
    // We really shouldn't be doing something destructive like a recursive
    // delete if we're encounting any problems.
    if (!file_valid_uri($packageUri) || !file_unmanaged_prepare($packageUri)) {
      $msg = t('Packager directory might not be configured correctly.');
      \Drupal::messenger()->addError($msg);
      return;
    }

    if (!file_unmanaged_delete_recursive($packageUri)) {
      $msg = t('Packager directory could be deleted.');
      \Drupal::messenger()->addError($msg);
    }

    if (!file_prepare_directory($packageUri, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
      $msg = t('Packager directory could not be prepared.');
      \Drupal::messenger()->addError($msg);
      return;
    }

    $context['message'] = t('Preparing package directory', []);
    $context['finished'] = 1.0;
  }

  /**
   * Implements callback_batch_operation().
   *
   * @param array $rows
   *   The view rows to process.
   * @param array $filters
   *   The image styles we want to generate URIs for and the fields we want
   *   to ignore.
   *     ['image_styles' => [], 'field_blacklist' => []].
   * @param array|\ArrayAccess $context
   *   The batch context array.
   */
  public static function findUrisFromViewRows(array $rows, array $filters, &$context) {
    if (!isset($context['sandbox']['progress'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['current_node'] = 0;
      $context['sandbox']['max'] = count($rows);
    }

    $uris = empty($context['results']['uris']) ? [] : $context['results']['uris'];
    /** @var \Drupal\views\ResultRow $row */
    $row = $rows[$context['sandbox']['progress']];
    $entity = $row->_entity;

    $uris = array_merge(EntityProcessor::processEntity($entity, $filters), $uris);
    $context['results']['uris'] = $uris;

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['message'] = t('@progress / @total view rows processed', [
      '@progress' => $context['sandbox']['progress'],
      '@total' => $context['sandbox']['max'],
    ]);

    if ($context['sandbox']['progress'] !== $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }

  /**
   * Implements callback_batch_operation().
   *
   * First renders a view and stores its rendered output onto the filesystem.
   *
   * @param string $view_id
   *   A view to render.
   * @param string $packageUri
   *   A URI destination.
   * @param array|\ArrayAccess $context
   *   The batch context array.
   */
  public static function renderAndSaveViewOutput($view_id, $packageUri, &$context) {
    if (!isset($context['sandbox']['next_stage'])) {
      $context['sandbox']['next_stage'] = 'render';
      $context['sandbox']['output'] = '';
    }

    if ($context['sandbox']['next_stage'] === 'render') {
      $view = Views::getView($view_id);

      // Find and use the first rest_export display.
      // "Default" display actually causes a crash!
      $displays = $view->storage->get('display');

      $display_id = NULL;

      foreach ($displays as $display) {
        if ($display['display_plugin'] === 'rest_export') {
          $display_id = $display['id'];
          break;
        }
      }

      if (!$display_id) {
        $context['results']['rendered'] = FALSE;
        $msg = t('Could not find an appropriate display to render!');
        \Drupal::messenger()->addError($msg);
        $context['finished'] = 1;
        return;
      }

      $view->setDisplay($display_id);
      $render_array = $view->preview();
      $context['message'] = t('Rendering view...');
      $context['sandbox']['output'] = $render_array['#markup'];
      $context['finished'] = 0.5;
      $context['sandbox']['next_stage'] = 'save';
      return;
    }

    // Must be time to save output!
    $output = $context['sandbox']['output'];
    $dest_dir = $packageUri . DIRECTORY_SEPARATOR;
    $filename = 'content.json';

    if (!file_put_contents($dest_dir . $filename, $output)) {
      $msg = t('Failed to output to file!');
      \Drupal::messenger()->addError($msg);
    }

    $context['message'] = t('Saving view output to disk.');
    $context['finished'] = 1;
    $context['results']['completed'][] = $dest_dir . $filename;
  }

  /**
   * Implements callback_batch_operation().
   *
   * Batch operation to duplicate files into the package folder.
   *
   * @param string $packageUri
   *   The URI that the packaging will be happening in.
   * @param array|\ArrayAccess $context
   *   The batch context array.
   */
  public static function copyImagesFromUris($packageUri, &$context) {
    if (!isset($context['sandbox']['progress'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['current_node'] = 0;
      $context['sandbox']['max'] = count($context['results']['uris']);
    }

    $uris = $context['results']['uris'];

    /** @var \Drupal\Core\File\FileSystem $filesystem */
    $filesystem = \Drupal::service('file_system');

    $source = $uris[$context['sandbox']['progress']];
    $source_uri = file_uri_target($source);
    $filename = $filesystem->basename($source_uri);
    $dirname = $filesystem->dirname($source_uri);
    $dest_dir = $packageUri . DIRECTORY_SEPARATOR . $dirname;

    $target = $dest_dir . DIRECTORY_SEPARATOR . $filename;

    if (!file_prepare_directory($dest_dir, FILE_CREATE_DIRECTORY | FILE_MODIFY_PERMISSIONS)) {
      \Drupal::logger('content_packager')->error('Failed to create the directory to pack images: %directory', ['%directory' => $dest_dir]);
    }
    if (!file_unmanaged_prepare($source, $target, FILE_EXISTS_REPLACE)) {
      $context['results']['failed'][] = $target;
    }
    else {
      $copy_result = file_unmanaged_copy($source, $target, FILE_EXISTS_REPLACE);
      $dt = filemtime($source);
      if ($dt !== FALSE && $copy_result !== TRUE) {
        touch($target, $dt);
      }
      if ($copy_result !== TRUE) {
        $context['results']['completed'][] = $target;
      }
    }

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['message'] = t('@progress / @total files copied', [
      '@progress' => $context['sandbox']['progress'],
      '@total' => $context['sandbox']['max'],
    ]);

    // Inform the batch engine that we are not finished,
    // and provide an estimation of the completion level we reached.
    if ($context['sandbox']['progress'] !== $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }

  /**
   * Implements callback_batch_operation().
   *
   * @param string $packageUri
   *   The URI that the packaging will be happening in.
   * @param array|\ArrayAccess $context
   *   The batch context array.
   */
  public static function zipFiles($packageUri, $zip_name, &$context) {
    if (!isset($context['sandbox']['progress'])) {
      $context['sandbox']['progress'] = 0;
      $context['sandbox']['max'] = count($context['results']['completed']);
    }

    $current_uri = $context['results']['completed'][$context['sandbox']['progress']];
    $str = '@' . preg_quote($packageUri, '@') . '@';
    $current_zip_path = preg_replace($str, '', $current_uri);
    $current_real_path = \Drupal::service('file_system')->realpath($current_uri);
    $zipfile_path = \Drupal::service('file_system')->realpath($packageUri);
    $zipfile_path .= DIRECTORY_SEPARATOR . $zip_name;

    if ($context['sandbox']['progress'] === 0 && file_exists($zipfile_path)) {
      file_unmanaged_delete($zipfile_path);
    }

    $zip = new \ZipArchive();
    $opened = $zip->open($zipfile_path, \ZipArchive::CREATE);
    if ($opened === TRUE) {
      $res = $zip->addFile($current_real_path, $current_zip_path);
      $zip->close();
      $context['results']['zipfile'] = $zipfile_path;
    }

    // Update our progress information.
    $context['sandbox']['progress']++;
    $context['message'] = t('@progress / @total files packaged.', [
      '@progress' => $context['sandbox']['progress'],
      '@total' => $context['sandbox']['max'],
    ]);

    // Inform the batch engine that we are not finished,
    // and provide an estimation of the completion level we reached.
    if ($context['sandbox']['progress'] !== $context['sandbox']['max']) {
      $context['finished'] = $context['sandbox']['progress'] / $context['sandbox']['max'];
    }
  }

  /**
   * Implements callback_batch_finished().
   */
  public static function packingFinished($success, $results, $operations) {
    if (!$success) {
      \Drupal::messenger()->addError('The content has not been properly packaged.');
      return;
    }

    if (!empty($results['failed'])) {
      $msg = t('Some files did not copy; please refer to the <a href="@system-log">error log</a> for more information.',
        ['@system-log' => './admin/reports/dblog']);
      \Drupal::messenger()->addError($msg);
      return;
    }

    $file_count = count($results['completed']);

    \Drupal::messenger()->addStatus(t('Content packaging successfull!'));
    \Drupal::messenger()->addStatus(t('%file_count files copied to content packager directory.  Content packaging successfull!', ['%file_count' => $file_count]));
  }

}
