<?php

namespace Drupal\content_packager\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManager;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides the content package generation form.
 *
 * @package Drupal\content_packager\Form
 */
class CreatePackage extends FormBase {

  private $entityTypeManager;

  /**
   * Class constructor.
   */
  public function __construct(ConfigFactoryInterface $config_factory, EntityTypeManager $typeManager) {
    $this->entityTypeManager = $typeManager;
    $this->setConfigFactory($config_factory);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'content_packager_create_package';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    $form = [];
    $config = $this->config('content_packager.settings');

    $available_views = content_packager_get_rest_export_views();

    $options = [];

    foreach ($available_views as $id => $display) {
      $options[$display[0]] = $display[0];
    }

    $package_uri = content_packager_package_uri();
    $zip_name = $config->get('zip_name');
    $this->buildExistingPackageInfo($package_uri, $zip_name, $form);

    $form['make_package'] = [
      '#type' => 'details',
      '#title' => t('Make a content package'),
      '#open' => TRUE,
    ];
    $form['make_package']['selected_view'] = [
      '#type' => 'select',
      '#title' => t('View to export'),
      '#default_value' => '',
      '#options' => $options,
    ];

    $form['make_package']['destination'] = [
      '#markup' => "<p>Package destination: $package_uri</p>",
    ];

    $form['actions']['#type'] = 'actions';
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Make a new package'),
      '#button_type' => 'primary',
    ];

    return $form;
  }

  private function buildExistingPackageInfo($package_uri, $zip_name, &$form) {
    $full_package_uri = $package_uri . DIRECTORY_SEPARATOR . $zip_name;
    if (!file_exists($full_package_uri)) {
      return;
    }

    $url = file_create_url($full_package_uri);

    $form['existing_file'] = [
      '#type' => 'details',
      '#title' => t('Current Package'),
      '#prefix' => t('A package already exists and <a href=":package_link">can be downloaded</a> at any time.',
        [':package_link' => $url]),
      '#open' => FALSE,
    ];

    $form['existing_file']['file'] = [
      '#type' => 'item',
      '#title' => 'File',
      '#markup' => t('<a href=":package_uri">%display_uri</a>',
        [
          ':package_uri' => $url,
          '%display_uri' => $full_package_uri,
        ]),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $view_name = $form_state->getValue('selected_view');
    $result = views_get_view_result($view_name);

    $package_uri = content_packager_package_uri();

    $image_styles = array_keys($this->config('content_packager.settings')->get('image_styles'));
    $field_blacklist = $this->config('content_packager.settings')->get('fields_ignored');
    $zip_name = $this->config('content_packager.settings')->get('zip_name');

    $prepare_folder = [
      'Drupal\content_packager\BatchOperations::prepareDestination',
      [$package_uri],
    ];

    $options = [
      'image_styles' => $image_styles,
      'field_blacklist' => $field_blacklist,
      'zip_name' => $zip_name,
    ];

    $find_uris_operation = [
      'Drupal\content_packager\BatchOperations::findUrisFromViewRows',
      [$result, $options],
    ];

    $view_render_operation = [
      'Drupal\content_packager\BatchOperations::renderAndSaveViewOutput',
      [$view_name, $package_uri],
    ];

    $file_copy_operation = [
      'Drupal\content_packager\BatchOperations::copyImagesFromUris',
      [$package_uri],
    ];

    $zip_operation = [
      'Drupal\content_packager\BatchOperations::zipFiles',
      [$package_uri, $zip_name],
    ];

    $batch = [
      'operations' => [
        $prepare_folder,
        $find_uris_operation,
        $view_render_operation,
        $file_copy_operation,
        $zip_operation,
      ],
      'finished' => 'Drupal\content_packager\BatchOperations::packingFinished',
    ];

    batch_set($batch);
  }

}
