<?php

namespace Drupal\Tests\content_packager\Functional;

use Drupal\content_packager\BatchOperations;
use Drupal\Tests\BrowserTestBase;
use Drupal\Tests\TestFileCreationTrait;
use Drupal\file\Entity\File;

/**
 * Provides methods specifically for testing Content Packager's configuration.
 *
 * @group content_packager
 */
class ContentPackagerDirectoryTest extends BrowserTestBase {

  use TestFileCreationTrait {
    getTestFiles as drupalGetTestFiles;
  }

  /**
   * Modules to enable.
   *
   * @var array
   */
  public static $modules = ['file', 'content_packager'];

  /**
   * An user with administration permissions.
   *
   * @var \Drupal\user\UserInterface
   */
  protected $user;

  /**
   * {@inheritdoc}
   */
  protected function setUp() {
    parent::setUp();
    $this->user = $this->drupalCreateUser([
      'access administration pages',
      'administer site configuration',
      'administer content packager',
      'create content package',
    ]);
    $this->drupalLogin($this->user);
  }

  /**
   * Retrieves a sample file of the specified type.
   *
   * @return \Drupal\file\FileInterface
   */
  public function getTestFile($type_name, $size = NULL) {
    // Get a file to upload.
    $file = current($this->drupalGetTestFiles($type_name, $size));

    // Add a filesize property to files as would be read by
    // \Drupal\file\Entity\File::load().
    $file->filesize = filesize($file->uri);

    return File::create((array) $file);
  }

  /**
   * Tests that destination config is working.
   */
  public function testDestinationConfig() {
    $this->drupalLogin($this->user);

    $this->drupalGet('admin/config/content/content_packager');
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalPostForm(NULL, array(
      'package_scheme' => 'public://',
      'package_destination' => 'test_folder',
    ), t('Save configuration'));

    $package_uri = content_packager_package_uri();
    $this->assertFileExists($package_uri, 'Directory exists.');
    $this->assertFileExists($package_uri . DIRECTORY_SEPARATOR . '.htaccess', 'Directory .htaccess exists.');

    $file = $this->getTestFile('image');

    /* @var  \Drupal\Core\File\FileSystem  $file_manager */
    $file_manager = \Drupal::service('file_system');

    $uri = $file->getFileUri();
    $uri_target = file_uri_target($uri);
    $batch_context = [
      'results' => ['uris' => [$uri]],
      'finished' => NULL,
      'sandbox' => [],
    ];

    BatchOperations::copyImagesFromUris($package_uri, $batch_context);
    $this->assertFileExists($package_uri . DIRECTORY_SEPARATOR . $uri_target, 'File copy exists where expected.');
  }

  /**
   * Ensure that the CreatePackage form correctly detects existing packages.
   */
  public function testPackageCreateFormDetection() {
    $this->drupalLogin($this->user);

    $file = $this->getTestFile('image');

    $uri = $file->getFileUri();
    $batch_context = [
      'results' => [
        'uris' => [$uri],
        'completed' => [],
      ],
      'finished' => NULL,
      'sandbox' => [],
    ];

    $package_uri = content_packager_package_uri();
    BatchOperations::copyImagesFromUris($package_uri, $batch_context);
    $batch_context['sandbox'] = [];

    $zip_name = \Drupal::config('content_packager.settings')->get('zip_name');
    BatchOperations::zipFiles($package_uri, $zip_name, $batch_context);

    $this->assertFileExists($package_uri . DIRECTORY_SEPARATOR . $zip_name, 'Zip file exists where expected.');

    $this->drupalGet('admin/content/package');
    $this->assertSession()->responseContains('A package already exists', 'Form indicates package already exists.');
  }

}
